@@ -69,7 +69,7 @@ static SCM sym_object = SCM_UNSPECIFIED;
 static SCM sym_args = SCM_UNSPECIFIED;
 static scm_t_bits dynproc_smob_tag = 0;
 
-#if 0 // not useed ATM
+#if 0 // not used ATM
 void
 gw_guile_runtime_get_version_info(int *major, int *revision, int *age)
 {
@@ -247,32 +247,26 @@ gw_guile_make_latent_variable (SCM sym, SCM proc, SCM arg)
 
 /* Grr. Seems subrs can't be methods. */
 static void 
-gw_guile_add_subr_method (SCM generic, SCM subr, SCM class_name, SCM module,
+gw_guile_add_subr_method (SCM generic, SCM subr, SCM req_specializers,
                           int n_req_args, int use_optional_args)
 {
   int i;
   char buffer[32];
   SCM specializers, formals, procm, meth, rest_sym = SCM_BOOL_F;
   
+  formals = SCM_EOL;
   if (use_optional_args)
   {
-    formals = SCM_EOL;
     rest_sym = scm_str2symbol ("rest");
-    specializers = scm_class_top;
+    specializers = scm_cons (req_specializers, scm_class_top);
   }
   else
-    formals = specializers = SCM_EOL;
+    specializers = req_specializers;
   
   for (i = n_req_args; i > 0; i--)
   {
     sprintf (buffer, "arg%d", i);
     formals = scm_cons (scm_str2symbol (buffer), formals);
-    if (i == 1)
-      specializers =
-        scm_cons (SCM_VARIABLE_REF (scm_module_lookup (module, class_name)),
-                  specializers);
-    else
-      specializers = scm_cons (scm_class_top, specializers);
   }
 
   if (use_optional_args)
@@ -333,7 +327,7 @@ gw_scm_module_binder_proc (SCM module, SCM sym, SCM definep)
     velts = SCM_VELTS (entry);
 
     gw_guile_add_subr_method (generic,
-                              velts[0], velts[1], velts[2],
+                              velts[0], velts[1],
                               SCM_INUM (velts[3]), SCM_NFALSEP (velts[4]));
 
     proc_list = SCM_CDR (proc_list);
@@ -372,7 +366,7 @@ ensure_scm_module_hacked (void)
 
 /* no explicit returns in this function */
 void
-gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
+gw_guile_procedure_to_method_public (SCM proc, SCM req_specializers,
                                      SCM generic_name,
                                      SCM n_req_args, SCM use_optional_args)
 #define FUNC_NAME "%gw:procedure-to-method-public!"
@@ -381,7 +375,7 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
   SCM existing_latents;
 
   SCM_VALIDATE_PROC (1, proc);
-  SCM_VALIDATE_SYMBOL (2, class_name);
+  SCM_VALIDATE_LIST (2, req_specializers);
   SCM_VALIDATE_SYMBOL (3, generic_name);
   SCM_VALIDATE_INUM (4, n_req_args);
   /* the fifth is a bool */
@@ -410,7 +404,7 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
     /* entry := #(proc class_name module n_req_args use_optional_args) */
     
     SCM_VECTOR_SET (entry, 0, proc);
-    SCM_VECTOR_SET (entry, 1, class_name);
+    SCM_VECTOR_SET (entry, 1, req_specializers);
     SCM_VECTOR_SET (entry, 2, scm_current_module ());
     SCM_VECTOR_SET (entry, 3, n_req_args);
     SCM_VECTOR_SET (entry, 4, use_optional_args);
@@ -466,7 +460,7 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
       scm_call_3 (module_add_x, the_root_module, generic_name,
                   scm_make_variable (generic));
     }
-    gw_guile_add_subr_method (generic, proc, class_name, scm_current_module(),
+    gw_guile_add_subr_method (generic, proc, req_specializers, 
                               SCM_INUM (n_req_args),
                               SCM_NFALSEP (use_optional_args));
   }
@@ -618,7 +612,25 @@ static void
 gw_guile_register_wrapset (GWWrapSet *ws)
 {
   int i;
+  SCM *classes;
+  
+  ws->lang_data = gw_malloc (ARENA, sizeof (SCM) * ws->ntypes);
 
+  /* Collect the classes, if given, of all types in the wrapset */
+  classes = (SCM *) ws->lang_data;
+  for (i = 0; i < ws->ntypes; i++)
+  {
+    GWTypeInfo *ti = &ws->types[i];
+    
+    if (ti->class_name)
+      classes[i] = SCM_VARIABLE_REF (scm_c_lookup (ti->class_name));
+    else
+      classes[i] = scm_class_top;
+    
+    scm_gc_protect_object (classes[i]);
+    ti->lang_data = &classes[i];
+  }
+  
   for (i = 0; i < ws->nfunctions; i++)
   {
     SCM subr;
@@ -649,11 +661,17 @@ gw_guile_register_wrapset (GWWrapSet *ws)
                                  use_extra_args, (SCM (*)())fi->proc);
     }
     
-    if (fi->generic_name && fi->arg_types && fi->arg_types[0]->class_name)
+    if (fi->generic_name && fi->arg_types)
     {
+      int j;
+      SCM specializers = SCM_EOL;
+      
+      for (j = fi->n_req_args - 1; j >= 0; j--)
+        specializers = scm_cons (*(SCM *) fi->arg_types[j]->lang_data,
+                                 specializers);
+      
       gw_guile_procedure_to_method_public (
-              subr,
-              scm_str2symbol (fi->arg_types[0]->class_name),
+              subr, specializers,
               scm_str2symbol (fi->generic_name),
               SCM_MAKINUM (fi->n_req_args),
               (fi->n_optional_args ? SCM_BOOL_T : SCM_BOOL_F));
